export class Main extends Indicator {

    constructor() {
        super();

        this.addInput("BandsPeriod", InputType.int, 20);
        this.addInput("BandsShift", InputType.int, 0);
        this.addInput("PeriodsATR", InputType.int, 20);
        this.addInput("PriceType", InputType.PriceType, PriceType.Bid);

        this.buffers = {
            MovingBuffer : this.addBuffer(),
            UpperBuffer : this.addBuffer(),
            UpperBuffer2 : this.addBuffer(),
            UpperBuffer3 : this.addBuffer(),
            LowerBuffer : this.addBuffer(),
            LowerBuffer2 : this.addBuffer(),
            LowerBuffer3 : this.addBuffer(),
        };
    }


    onInit() {
        let {BandsPeriod, BandsShift} = this.getInputs(),
            drawBegin = BandsPeriod + BandsShift;

        this.buffers.MovingBuffer
            .setShape(Shape.Line)
            .setColor(Color.Blue)
            .setDrawingStyle(DrawingStyle.Dot)
            .setDrawBegin(drawBegin)
        ;

        this.buffers.UpperBuffer
            .setShape(Shape.Line)
            .setColor(Color.Green)
            .setDrawingStyle(DrawingStyle.Solid)
            .setDrawBegin(drawBegin)
        ;

        this.buffers.UpperBuffer2
            .setShape(Shape.Line)
            .setColor(Color.Green)
            .setDrawingStyle(DrawingStyle.Solid)
            .setDrawBegin(drawBegin)
        ;

        this.buffers.UpperBuffer3
            .setShape(Shape.Line)
            .setColor(Color.Red)
            .setDrawingStyle(DrawingStyle.Solid)
            .setDrawBegin(drawBegin)
        ;

        this.buffers.LowerBuffer
            .setShape(Shape.Line)
            .setColor(Color.LightSeaGreen)
            .setDrawingStyle(DrawingStyle.Solid)
            .setWidth(2)
            .setDrawBegin(drawBegin)
        ;

        this.buffers.LowerBuffer2
            .setShape(Shape.Line)
            .setColor(Color.Red)
            .setDrawingStyle(DrawingStyle.Solid)
            .setDrawBegin(drawBegin)
        ;

        this.buffers.LowerBuffer3
            .setShape(Shape.Line)
            .setColor(Color.LightSeaGreen)
            .setDrawingStyle(DrawingStyle.Solid)
            .setWidth(2)
            .setDrawBegin(drawBegin)
        ;
    }

    async onUpdate() {
        const {BandsPeriod, BandsShift, PeriodsATR, PriceType} = this.getInputs(),
            {MovingBuffer, UpperBuffer, UpperBuffer2, UpperBuffer3, LowerBuffer, LowerBuffer2, LowerBuffer3} = this.buffers,
            {
                High,
                Low,
                Close,
            } = await Bar.load([
                Bar.Mode.High,
                Bar.Mode.Low,
                Bar.Mode.Close,
            ], PriceType),
            barSize = Close.length;

        let i, k, deviation, sum, oldval, limit, list,
            EMPTY_VALUE = 0,
            nCountedBars = Bar.counted();

        if (barSize <= BandsPeriod) {
            return 0;
        }
        if (nCountedBars < 1)
            for (i = 1; i <= BandsPeriod; i++) {
                MovingBuffer.set((barSize - i), EMPTY_VALUE);
                UpperBuffer.set((barSize - i), EMPTY_VALUE);
                UpperBuffer2.set((barSize - i), EMPTY_VALUE);
                UpperBuffer3.set((barSize - i), EMPTY_VALUE);
                LowerBuffer.set((barSize - i), EMPTY_VALUE);
                LowerBuffer2.set((barSize- i), EMPTY_VALUE);
                LowerBuffer3.set((barSize - i), EMPTY_VALUE);
            }
        limit = barSize - nCountedBars;
        if (nCountedBars > 0) limit++;

        list = await Indicators.MA(null, 0, BandsPeriod, BandsShift, Averaging.Simple, AppliedPrice.Close, PriceType);

        for (i = 0; i < limit; i++) {
            MovingBuffer.set(i, list[BandsShift+i]);
        }
        i = barSize - BandsPeriod + 1;
        if (nCountedBars > BandsPeriod - 1) {
            i = barSize - nCountedBars - 1;
        }
        list = await Indicators.ATR(null, 0, PeriodsATR, PriceType);
        while (i >= 0) {
            sum = 0.0;
            k = i + BandsPeriod - 1;
            oldval = MovingBuffer.get(i);
            deviation = list[i];
            UpperBuffer.set(i, (oldval + deviation * 1.618));
            UpperBuffer2.set(i, (oldval + deviation * 2.618));
            UpperBuffer3.set(i, (oldval + deviation * 4.236));
            LowerBuffer.set(i, (oldval - deviation * 1.618));
            LowerBuffer2.set(i, (oldval - deviation * 2.618));
            LowerBuffer3.set(i, (oldval - deviation * 4.236));
            i--;
        }
    }
}